@@ -19,17 +19,19 @@
  *  along with this program ; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  *
- * $Id: CXvidDecoder.h 2059 2012-02-22 19:00:26Z Isibaar $
+ * $Id: CXvidDecoder.h 2114 2015-06-14 19:18:14Z Isibaar $
  *
  ****************************************************************************/
 
 #ifndef _FILTER_H_
 #define _FILTER_H_
 
+#include <time.h>
 #include <xvid.h>
 #include "IXvidDecoder.h"
 
 #define XVID_NAME_L		L"Xvid MPEG-4 Video Decoder"
+#define XVID_NAME_MFT_L	L"Xvid MPEG-4 Video Decoder MFT"
 
 /* --- fourcc --- */
 
@@ -153,7 +155,7 @@ public :
 
 private :
 
-	HRESULT ChangeColorspace(GUID subtype, GUID formattype, void * format, int noflip);
+	HRESULT ChangeColorspace(GUID subtype, GUID formattype, void * format, int *bitdepth, int noflip);
 	HRESULT OpenLib();
 	void CloseLib();
 
@@ -169,6 +171,9 @@ private :
 	int rgb_flip;
 	int out_stride;
 
+	clock_t m_startClock;
+	int m_tray_icon;
+
 	/* mft stuff */
 #if defined(XVID_USE_MFT)
 	BOOL HasPendingOutput() const { return m_frame.output.plane[1] != NULL; }
@@ -176,10 +181,11 @@ private :
 	HRESULT OnSetInputType(IMFMediaType *pmt);
 	HRESULT OnCheckInputType(IMFMediaType *pmt);
 
-	HRESULT OnSetOutputType(IMFMediaType *pmt);
+	HRESULT OnSetOutputType(IMFMediaType *pmt, int bitdepth);
 
 	IMFMediaType *m_pInputType;
 	IMFMediaType *m_pOutputType;
+	int m_pOutputTypeBPP;
 
 	CRITICAL_SECTION m_mft_lock;
 	REFERENCE_TIME m_timestamp;
@@ -191,9 +197,9 @@ private :
 	REFERENCE_TIME m_rtFrame;
 	MFRatio m_frameRate;
 	UINT64 m_duration;
-#endif
 
-	HWND MSG_hwnd; /* message handler window */
+	HANDLE m_thread_handle;
+#endif
 };
 #define WM_ICONMESSAGE (WM_USER + 1)
 
